@@ -1,18 +1,28 @@
+import operator
 import re
+import warnings
+
+try:
+    from collections.abc import MutableMapping
+except ImportError:  # Python 2
+    from collections import MutableMapping
+
 
 __package__ = 'python-intervals'
-__version__ = '1.8.0'
+__version__ = '1.9.0'
 __licence__ = 'LGPL3'
 __author__ = 'Alexandre Decan'
 __url__ = 'https://github.com/AlexandreDecan/python-intervals'
-__description__ = 'Interval arithmetic for Python'
+__description__ = 'Python data structure and operations for intervals'
 
 
 __all__ = [
     'inf', 'CLOSED', 'OPEN',
     'Interval',  # 'AtomicInterval',
     'open', 'closed', 'openclosed', 'closedopen', 'singleton', 'empty',
+    'iterate',
     'from_string', 'to_string', 'from_data', 'to_data',
+    'IntervalDict',
 ]
 
 
@@ -121,8 +131,9 @@ def empty():
     return empty._instance
 
 
-def from_string(string, conv, bound=r'.+?', disj=r' ?\| ?', sep=r', ?', left_open=r'\(',
-                left_closed=r'\[', right_open=r'\)', right_closed=r'\]', pinf=r'\+inf', ninf=r'-inf'):
+def from_string(string, conv, bound=r'.+?', disj=r' ?\| ?', sep=r', ?',
+                left_open=r'\(', left_closed=r'\[', right_open=r'\)', right_closed=r'\]',
+                pinf=r'\+inf', ninf=r'-inf'):
     """
     Parse given string and create an Interval instance.
     A converter function has to be provided to convert a bound (as string) to a value.
@@ -230,7 +241,7 @@ def from_data(data, conv=None, pinf=float('inf'), ninf=float('-inf')):
     Import an interval from a piece of data.
 
     :param data: a list of 4-uples (left, lower, upper, right).
-    :param conv: function that is used to convert "lower" and "upper" to bounds, default to identity.
+    :param conv: function that converts "lower" and "upper" to bounds, default to identity.
     :param pinf: value used to represent positive infinity.
     :param ninf: value used to represent negative infinity.
     :return: an Interval instance.
@@ -291,6 +302,56 @@ def to_data(interval, conv=None, pinf=float('inf'), ninf=float('-inf')):
     return data
 
 
+def iterate(interval, incr, base=None, reverse=False):
+    """
+    Iterate on the (discrete) values of given interval.
+
+    This function returns a (lazy) iterator over the values of given (atomic or not) interval,
+    starting from its lower bound and ending on its upper bound (if interval is not open).
+    Each returned value merely corresponds to lower + i * incr, where "incr" defines
+    the step between consecutive values. This parameter must be a (positive) value, even if
+    "reverse" is set to True. It also accepts a callable that is used to compute the next possible
+    value based on the current one.
+
+    When a non-atomic interval is provided, this function chains the iterators obtained
+    by calling itself on the underlying atomic intervals.
+
+    The values returned by the iterator can be aligned with a base value with the "base" parameter.
+    This parameter must be a callable that accepts the lower bound of the (atomic) interval as
+    input, and returns the first value that needs to be considered for the iteration.
+    By default, the identity function is used. If reverse=True, then the upper bound will be
+    passed instead of the lower one.
+
+    :param interval: Interval or atomic interval.
+    :param incr: (positive) step between values, or a callable that returns the next value.
+    :param base: a callable that accepts a bound and returns an initial value to consider.
+    :param reverse: Set to True for descending order.
+    :return: A lazy iterator.
+    """
+    intervals = [interval] if isinstance(interval, AtomicInterval) else interval._intervals
+
+    if base is None:
+        base = (lambda x: x)
+
+    exclude = operator.lt if not reverse else operator.gt
+    include = operator.le if not reverse else operator.ge
+    increment = incr if callable(incr) else (lambda x: x + incr if not reverse else x - incr)
+
+    value = base(interval.lower if not reverse else interval.upper)
+    if (value == -inf and not reverse) or (value == inf and reverse):
+        raise ValueError('Cannot start iteration with infinity.')
+
+    for i in intervals if not reverse else reversed(intervals):
+        value = base(i.lower if not reverse else i.upper)
+
+        while exclude(value, i):
+            value = increment(value)
+
+        while include(value, i):
+            yield value
+            value = increment(value)
+
+
 class AtomicInterval:
     """
     This class represents an atomic interval.
@@ -308,10 +369,10 @@ class AtomicInterval:
         If a bound is set to infinity (regardless of its sign), the corresponding boundary will
         be exclusive.
 
-        :param left: Boolean indicating whether the left boundary is inclusive (True) or exclusive (False).
-        :param lower: lower bound value.
-        :param upper: upper bound value.
-        :param right: Boolean indicating whether the right boundary is inclusive (True) or exclusive (False).
+        :param left: Boolean indicating if left boundary is inclusive (True) or exclusive (False).
+        :param lower: value of the lower bound.
+        :param upper: value of the upper bound.
+        :param right: Boolean indicating if right boundary is inclusive (True) or exclusive (False).
         """
         self._left = left if lower not in [inf, -inf] else OPEN
         self._lower = lower
@@ -400,27 +461,32 @@ class AtomicInterval:
 
         return AtomicInterval(left, lower, upper, right)
 
-    def overlaps(self, other, permissive=False):
+    def overlaps(self, other, adjacent=False, permissive=None):
         """
         Test if intervals have any overlapping value.
 
-        If 'permissive' is set to True (default is False), then [1, 2) and [2, 3] are considered as having
-        an overlap on value 2 (but not [1, 2) and (2, 3]).
+        If 'adjacent' is set to True (default is False), then it returns True for adjacent
+        intervals as well (e.g., [1, 2) and [2, 3], but not [1, 2) and (2, 3]).
 
         :param other: an atomic interval.
-        :param permissive: set to True to consider contiguous intervals as well.
-        :return True if intervals overlap, False otherwise.
+        :param adjacent: set to True to accept adjacent intervals as well.
+        :param permissive: deprecated, use adjacent instead.
+        :return: True if intervals overlap, False otherwise.
         """
+        if permissive is not None:
+            adjacent = permissive
+            warnings.warn('`permissive` is deprecated, use `adjacent` instead.', DeprecationWarning)
+
         if not isinstance(other, AtomicInterval):
             raise TypeError('Only AtomicInterval instances are supported.')
 
-        if self._lower > other.lower:
-            first, second = other, self
-        else:
+        if self._lower < other.lower or (self._lower == other.lower and self._left == CLOSED):
             first, second = self, other
+        else:
+            first, second = other, self
 
         if first._upper == second._lower:
-            if permissive:
+            if adjacent:
                 return first._right == CLOSED or second._left == CLOSED
             else:
                 return first._right == CLOSED and second._left == CLOSED
@@ -438,8 +504,9 @@ class AtomicInterval:
 
     def union(self, other):
         """
-        Return the union of two intervals. If the union cannot be represented using a single atomic interval,
-        return an Interval instance (which corresponds to an union of atomic intervals).
+        Return the union of two intervals. If the union cannot be represented using a single
+        atomic interval, return an Interval instance (which corresponds to an union of atomic
+        intervals).
 
         :param other: an interval.
         :return: the union of the intervals.
@@ -501,17 +568,17 @@ class AtomicInterval:
 
     def __or__(self, other):
         if isinstance(other, AtomicInterval):
-            if self.overlaps(other, permissive=True):
+            if self.overlaps(other, adjacent=True):
                 if self._lower == other._lower:
                     lower = self._lower
-                    left = self._left if self._left == OPEN else other._left
+                    left = self._left if self._left == CLOSED else other._left
                 else:
                     lower = min(self._lower, other._lower)
                     left = self._left if lower == self._lower else other._left
 
                 if self._upper == other._upper:
                     upper = self._upper
-                    right = self._right if self._right == OPEN else other._right
+                    right = self._right if self._right == CLOSED else other._right
                 else:
                     upper = max(self._upper, other._upper)
                     right = self._right if upper == self._upper else other._right
@@ -525,9 +592,11 @@ class AtomicInterval:
     def __contains__(self, item):
         if isinstance(item, AtomicInterval):
             left = item._lower > self._lower or (
-                    item._lower == self._lower and (item._left == self._left or self._left == CLOSED))
+                item._lower == self._lower and (item._left == self._left or self._left == CLOSED)
+            )
             right = item._upper < self._upper or (
-                    item._upper == self._upper and (item._right == self._right or self._right == CLOSED))
+                item._upper == self._upper and (item._right == self._right or self._right == CLOSED)
+            )
             return left and right
         elif isinstance(item, Interval):
             for interval in item:
@@ -573,44 +642,48 @@ class AtomicInterval:
             if self._right == OPEN:
                 return self._upper <= other._lower
             else:
-                return self._upper < other._lower or (self._upper == other._lower and other._left == OPEN)
+                return self._upper < other._lower or \
+                    (self._upper == other._lower and other._left == OPEN)
         elif isinstance(other, Interval):
             return self < other.to_atomic()
         else:
-            return AtomicInterval(CLOSED, other, other, CLOSED) > self
+            return self._upper < other or (self._right == OPEN and self._upper == other)
 
     def __gt__(self, other):
         if isinstance(other, AtomicInterval):
             if self._left == OPEN:
                 return self._lower >= other._upper
             else:
-                return self._lower > other._upper or (self._lower == other._upper and other._right == OPEN)
+                return self._lower > other._upper or \
+                    (self._lower == other._upper and other._right == OPEN)
         elif isinstance(other, Interval):
             return self > other.to_atomic()
         else:
-            return AtomicInterval(CLOSED, other, other, CLOSED) < self
+            return self._lower > other or (self._left == OPEN and self._lower == other)
 
     def __le__(self, other):
         if isinstance(other, AtomicInterval):
             if self._right == OPEN:
                 return self._upper <= other._upper
             else:
-                return self._upper < other._upper or (self._upper == other._upper and other._right == CLOSED)
+                return self._upper < other._upper or \
+                    (self._upper == other._upper and other._right == CLOSED)
         elif isinstance(other, Interval):
             return self <= other.to_atomic()
         else:
-            return AtomicInterval(CLOSED, other, other, CLOSED) >= self
+            return self._lower < other or (self._left == CLOSED and self._lower == other)
 
     def __ge__(self, other):
         if isinstance(other, AtomicInterval):
             if self._left == OPEN:
                 return self._lower >= other._lower
             else:
-                return self._lower > other._lower or (self._lower == other._lower and other._left == CLOSED)
+                return self._lower > other._lower or \
+                    (self._lower == other._lower and other._left == CLOSED)
         elif isinstance(other, Interval):
             return self >= other.to_atomic()
         else:
-            return AtomicInterval(CLOSED, other, other, CLOSED) <= self
+            return self._upper > other or (self._right == CLOSED and self._upper == other)
 
     def __hash__(self):
         try:
@@ -675,10 +748,10 @@ class Interval:
                 current = self._intervals[i]
                 successor = self._intervals[i + 1]
 
-                if current.overlaps(successor, permissive=True):
+                if current.overlaps(successor, adjacent=True):
                     interval = current | successor
-                    self._intervals.pop(i)
-                    self._intervals.pop(i)
+                    self._intervals.pop(i)  # pop current
+                    self._intervals.pop(i)  # pop successor
                     self._intervals.insert(i, interval)
                 else:
                     i = i + 1
@@ -686,7 +759,8 @@ class Interval:
     @property
     def left(self):
         """
-        Boolean indicating whether the lowest left boundary is inclusive (True) or exclusive (False).
+        Boolean indicating whether the lowest left boundary is inclusive (True) or
+        exclusive (False).
         """
         return self._intervals[0].left
 
@@ -707,7 +781,8 @@ class Interval:
     @property
     def right(self):
         """
-        Boolean indicating whether the highest right boundary is inclusive (True) or exclusive (False).
+        Boolean indicating whether the highest right boundary is inclusive (True)
+        or exclusive (False).
         """
         return self._intervals[-1].right
 
@@ -721,7 +796,8 @@ class Interval:
 
     def is_atomic(self):
         """
-        Test interval atomicity. An interval is atomic if it is composed of a single atomic interval.
+        Test interval atomicity. An interval is atomic if it is composed of a
+        single atomic interval.
 
         :return: True if this interval is atomic, False otherwise.
         """
@@ -742,8 +818,8 @@ class Interval:
 
     def enclosure(self):
         """
-        Return the smallest interval composed of a single atomic interval that encloses the current interval.
-        Is equivalent to Interval(self.to_atomic())
+        Return the smallest interval composed of a single atomic interval that encloses
+        the current interval. This method is equivalent to Interval(self.to_atomic())
 
         :return: an Interval instance.
         """
@@ -776,12 +852,18 @@ class Interval:
             left = enclosure._left if left is None else left
 
         if callable(lower):
-            lower = enclosure._lower if ignore_inf and enclosure._lower in [-inf, inf] else lower(enclosure._lower)
+            if ignore_inf and enclosure._lower in [-inf, inf]:
+                lower = enclosure._lower
+            else:
+                lower = lower(enclosure._lower)
         else:
             lower = enclosure._lower if lower is None else lower
 
         if callable(upper):
-            upper = enclosure._upper if ignore_inf and enclosure._upper in [-inf, inf] else upper(enclosure._upper)
+            if ignore_inf and enclosure._upper in [-inf, inf]:
+                upper = enclosure._upper
+            else:
+                upper = upper(enclosure._upper)
         else:
             upper = enclosure._upper if upper is None else upper
 
@@ -822,25 +904,30 @@ class Interval:
 
         return Interval(*intervals)
 
-    def overlaps(self, other, permissive=False):
+    def overlaps(self, other, adjacent=False, permissive=None):
         """
         Test if intervals have any overlapping value.
 
-        If 'permissive' is set to True (default is False), then [1, 2) and [2, 3] are considered as having
-        an overlap on value 2 (but not [1, 2) and (2, 3]).
+        If 'adjacent' is set to True (default is False), then it returns True for adjacent
+        intervals as well (e.g., [1, 2) and [2, 3], but not [1, 2) and (2, 3]).
 
         :param other: an interval or atomic interval.
-        :param permissive: set to True to consider contiguous intervals as well.
-        :return True if intervals overlap, False otherwise.
+        :param adjacent: set to True to accept adjacent intervals as well.
+        :param permissive: deprecated, use adjacent instead.
+        :return: True if intervals overlap, False otherwise.
         """
+        if permissive is not None:
+            adjacent = permissive
+            warnings.warn('`permissive` is deprecated, use `adjacent` instead.', DeprecationWarning)
+
         if isinstance(other, AtomicInterval):
             for interval in self._intervals:
-                if interval.overlaps(other, permissive=permissive):
+                if interval.overlaps(other, adjacent=adjacent):
                     return True
             return False
         elif isinstance(other, Interval):
             for o_interval in other._intervals:
-                if self.overlaps(o_interval, permissive=permissive):
+                if self.overlaps(o_interval, adjacent=adjacent):
                     return True
             return False
         else:
@@ -989,3 +1076,281 @@ class Interval:
 
     def __repr__(self):
         return ' | '.join(repr(i) for i in self._intervals)
+
+
+class IntervalDict(MutableMapping):
+    """
+    An IntervalDict is a dict-like data structure that maps from intervals to data,
+    where keys can be single values or Interval instances.
+
+    When Interval instances are used as keys, its behaviour merely corresponds to
+    range queries and it returns IntervalDict instances corresponding to the
+    subset of values covered by the interval. When single values are used as keys,
+    its behaviour corresponds to built-in dict.
+
+    KeyError are only raised when values are used as keys. When intervals are used,
+    and if there is no matching value, an empty IntervalDict is returned instead.
+
+    Note that this class does not aim to have the best performance, but is
+    provided mainly for convenience.
+    """
+
+    __slots__ = ('_items', )
+
+    def __init__(self, mapping_or_iterable=None):
+        """
+        Return a new IntervalDict.
+
+        If no argument is given, an empty IntervalDict is created. If an argument
+        is given, and is a mapping object (e.g., another IntervalDict), an
+        new IntervalDict with the same key-value pairs is created.
+
+        If an iterable is provided, it has to be a list of (key, value) pairs.
+
+        :param mapping_or_iterable: optional mapping or iterable.
+        """
+        self._items = list()  # List of (interval, value) pairs
+
+        if mapping_or_iterable is not None:
+            self.update(mapping_or_iterable)
+
+    def clear(self):
+        """
+        Remove all items from the IntervalDict.
+        """
+        self._items = list()
+
+    def copy(self):
+        """
+        Return a shallow copy.
+
+        :return: a shallow copy.
+        """
+        return IntervalDict(self)
+
+    def get(self, key, default=None):
+        """
+        Return the values associated to given key.
+
+        If the key is a single value, it returns a single value (if it exists) or
+        the default value. If the key is an Interval, it returns a new IntervalDict
+        restricted to given interval. In that case, the default value is used to
+        "fill the gaps" (if any) w.r.t. given key.
+
+        :param key: a single value or an Interval instance.
+        :param default: default value (default to None).
+        :return: an IntervalDict, or a single value if key is not an Interval.
+        """
+        if isinstance(key, Interval):
+            d = self[key]
+            d[key - d.domain()] = default
+            return d
+        else:
+            try:
+                return self[key]
+            except KeyError:
+                return default
+
+    def find(self, value):
+        """
+        Return a (possibly empty) Interval i such that self[i] = value, and
+        self[~i] != value.
+
+        :param value: value to look for.
+        :return: an Interval instance.
+        """
+        return Interval(*(i for i, v in self._items if v == value))
+
+    def items(self):
+        """
+        Return the list of (Interval, value) pairs.
+
+        :return: a list of 2-uples.
+        """
+        def func(i):
+            return (i[0].lower, not i[0].left, i[0].upper, i[0].right)
+
+        return sorted(self._items, key=func)
+
+    def keys(self):
+        """
+        Return the list of underlying Interval instances.
+
+        :return: a list of intervals.
+        """
+        return [i for i, v in self.items()]
+
+    def values(self):
+        """
+        Return the list of values.
+
+        :return: a list of values.
+        """
+        return [v for i, v in self.items()]
+
+    def domain(self):
+        """
+        Return an Interval corresponding to the domain of this IntervalDict.
+
+        :return: an Interval.
+        """
+        return Interval(*(i for i, v in self._items))
+
+    def pop(self, key, default=None):
+        """
+        Return and remove given key.
+
+        This method combines self[key] and del self[key]. If a default value
+        is provided and is not None, it uses self.get(key, default) instead of
+        self[key].
+
+        :param key: a single value or an Interval instance.
+        :param default: optional default value.
+        :return: an IntervalDict, or a single value if key is not an Interval.
+        """
+        if default is None:
+            value = self[key]
+            del self[key]
+            return value
+        else:
+            value = self.get(key, default)
+            try:
+                del self[key]
+            except KeyError:
+                pass
+            return value
+
+    def popitem(self):
+        """
+        Pop an arbitrary existing key.
+
+        :return: an IntervalDict
+        """
+        try:
+            return self.pop(self._items[-1][0])
+        except IndexError:
+            raise KeyError('Instance is empty.')
+
+    def setdefault(self, key, default=None):
+        """
+        Return given key. If it does not exist, set its value to default and
+        return it.
+
+        :param key: a single value or an Interval instance.
+        :param default: default value (default to None).
+        :return: an IntervalDict, or a single value if key is not an Interval.
+        """
+        if isinstance(key, Interval):
+            value = self.get(key, default)
+            self.update(value)
+            return value
+        else:
+            try:
+                return self[key]
+            except KeyError:
+                self[key] = default
+                return default
+
+    def update(self, mapping_or_iterable):
+        """
+        Update current IntervalDict with provided values.
+
+        If a mapping is provided, it must map Interval instances to values (e.g., another
+        IntervalDict). If an iterable is provided, it must consist of a list of
+        (key, value) pairs.
+
+        :param mapping_or_iterable: mapping or iterable.
+        """
+        if hasattr(mapping_or_iterable, 'items'):
+            data = mapping_or_iterable.items()
+        else:
+            data = mapping_or_iterable
+
+        for i, v in data:
+            i = singleton(i) if not isinstance(i, Interval) else i
+            self[i] = v
+
+    def __getitem__(self, key):
+        if isinstance(key, Interval):
+            items = []
+            for i, v in self._items:
+                intersection = key & i
+                if not intersection.is_empty():
+                    items.append((intersection, v))
+            return IntervalDict(items)
+        else:
+            for i, v in self._items:
+                if key in i:
+                    return v
+            raise KeyError(key)
+
+    def __setitem__(self, key, value):
+        interval = key if isinstance(key, Interval) else singleton(key)
+
+        if interval.is_empty():
+            return
+
+        new_items = []
+        found = False
+        for i, v in self._items:
+            if value == v:
+                found = True
+                new_items.append((i | interval, v))
+            elif i.overlaps(interval):
+                remaining = i - interval
+                if not remaining.is_empty():
+                    new_items.append((remaining, v))
+            else:
+                new_items.append((i, v))
+
+        if not found:
+            new_items.append((interval, value))
+
+        self._items = new_items
+
+    def __delitem__(self, key):
+        interval = key if isinstance(key, Interval) else singleton(key)
+
+        if interval.is_empty():
+            return
+
+        new_items = []
+        found = False
+        for i, v in self._items:
+            if i.overlaps(interval):
+                found = True
+                remaining = i - interval
+                if not remaining.is_empty():
+                    new_items.append((remaining, v))
+            else:
+                new_items.append((i, v))
+
+        self._items = new_items
+
+        if not found and not isinstance(key, Interval):
+            raise KeyError(key)
+
+    def __iter__(self):
+        return iter(self.keys())
+
+    def __len__(self):
+        return len(self._items)
+
+    def __contains__(self, key):
+        return key in self.domain()
+
+    def __repr__(self):
+        return '{}{}{}'.format(
+            '{',
+            ', '.join('{!r}: {!r}'.format(i, v) for i, v in self.items()),
+            '}',
+        )
+
+    def __eq__(self, other):
+        if isinstance(other, IntervalDict):
+            return self.items() == other.items()
+        else:
+            return NotImplemented
+
+    def __ne__(self, other):
+        return not self == other  # Required for Python 2
